#!/usr/bin/python

__doc__ = "POP Account Base Classes"
__version__ = "0.0.2"

import string

class POPFolderBase:
	"""
	Mix-in Class for POP Account Holder Instances
	"""

	def createAccount(user, passwd, server):
		pass # do this later, for now use management interface


class POPMailAccountBase:
    def __init__(self, user='noname', passwd='nopass', server='noserver'):
	  self.user = user
	  self.passwd = passwd
	  self.server = server

	  self.MessageDict = {}
	  self.uidDict = {}

	  self._v_sock = None


########
#
# These next three are the most basic tools that all the others need
#
#######
    def Connect(self):
	  import poplib

	#
	# At some point we need to handle password errors here better
	#
	  self._v_sock = poplib.POP3(self.server)
	  self._v_sock.user(self.user)
	  self._v_sock.pass_(self.passwd)


    def UpdateUIDList(self):
	  response, uid_list, octets = self._v_sock.uidl() 

	  if string.upper(string.strip(response))[:3] == '+OK':
	  	self.uidDict = {}

	  	for uid_item in uid_list:
			id, uid = string.split(uid_item)
			self.uidDict[uid] = id


    def CheckStatus(self):
	  try:
		self._v_sock.noop()

	  except:
		self.Connect()


    def UpdateStatus(self):
	  try:
		self.UpdateUIDList()
	  except:
		self.Connect()
		self.UpdateUIDList()

########

    def ParseMessageHeader(self, message_uid):
	#
	# This should only be called when a message is not in our Dict
	#
	if not self.uidDict.has_key(message_uid):
		self.UpdateStatus()

	resultcode, messageheader, octets = self._v_sock.top(self.uidDict[message_uid], 0)
	header_length = len(string.join(messageheader))

	resultcode, message_id, message_size = string.split( self._v_sock.list(self.uidDict[message_uid]) )

	message_size = int(message_size)/1024
	if message_size < 1:
		message_size = 1

	message_size = "%s Kb" % message_size

	fromline = "none"
	toline = "none"
	sline = "none"
	dateline = "none"
	unread = 1

	for mline in messageheader:
	   if string.find(string.lower(mline), "from:") == 0:
		fromline = string.join(string.split(mline, ":")[1:])
		#fromline = string.replace(fromline, "<", "&lt;")
		#fromline = string.replace(fromline, ">", "&gt;")

	   elif string.find(string.lower(mline), "to:") == 0:
		toline = string.join(string.split(mline, ":")[1:])
		toline = string.replace(toline, "<", "&lt;")
		toline = string.replace(toline, ">", "&gt;")

	   elif string.find(string.lower(mline), "subject:") == 0:
		sline = string.join(string.split(mline, ":")[1:])

	   elif string.find(string.lower(mline), "date:") == 0:
		dateline = string.join(string.split(mline, ":")[1:])

	message = { "message_id"   : message_id,
		    "message_uid"  : message_uid,
		    "message_size" : message_size,
		    "header_len"   : header_length,
		    "from"         : fromline,
		    "to"           : toline,
		    "subject"      : sline,
		    "date"         : dateline,
		    "unread"	   : unread,
		  }

	self.MessageDict[message_uid] = message
	return message


    def ListMessages(self, mid=''):
	  returnMessages = []

	  self.UpdateStatus()

	  if mid != '':
		returnMessages.append( self.GetFullMessage(mid) )

	  else:
	  	for message_uid in self.uidDict.keys():
			returnMessages.append( self.GetMessage(message_uid) )

	  return returnMessages


    def GetMessage(self, message_uid):
	#
	# Gets everything but the body of the message.  That is only retrieved
	# if the user requests the message (for performance and size reasons).
	# The GetFullMessage method adds the body to the message
	#

	if self.MessageDict.has_key(message_uid):
	  return self.MessageDict[message_uid]

	else:

	  self.CheckStatus()

	  message = self.ParseMessageHeader(message_uid)

	  return message

    def GetFullMessage(self, message_uid):

	if not self.MessageDict.has_key(message_uid):
		message = self.GetMessage(message_uid)

	# Mark the message as having been read
	#self.MessageDict[message_uid].unread = 0


	if self.MessageDict[message_uid].has_key('body'):
		return self.MessageDict[message_uid]

	else:
		self.CheckStatus()
		status, messagebody, octets = self._v_sock.retr(self.uidDict[message_uid])
		messagebody = string.join(messagebody, "\n")
		messagebody = messagebody[self.MessageDict[message_uid]['header_len']:]
		messagebody = string.replace(messagebody, "<", "&lt;")
		messagebody = string.replace(messagebody, ">", "&gt;")
		messagebody = string.replace(messagebody, "\n", "<BR>\n")

		self.MessageDict[message_uid]['body'] = messagebody
		return self.MessageDict[message_uid]

		

    def DeleteMessage(self, message_uid):
	  	self.UpdateStatus()

	  	messageID = self.uidDict[message_uid]

	  #try:
		returnstring = "Deleting in socket\n"

	  	self._v_sock.dele(messageID)

		returnstring = returnstring + "socket message %s deleted" % messageID

		del self.MessageDict[message_uid]

		returnstring = returnstring + "\nMessageDict updated"

	  	self.UpdateStatus()

		return returnstring

		return 1 # deleted successfully

	  #except:
		return 0 # delete failed for some reason


#
# These next few are not really needed right now but may be later
#
    def Status(self):
	  self.CheckStatus()

	  messagecount, mailboxsize = self._v_sock.stat()

	  return {'MessageCount':messagecount, 'MailboxSize':mailboxsize }


    def ServerStatus(self):
	  self.CheckStatus()

	  result = self._v_sock.getwelcome()
	  return result


    def ResetMailbox(self):
	#
	# Remove any deletion marks from messages
	#
	  self.CheckStatus()

	  self._v_sock.rset()


    def CloseSocket(self):
	  try:
	  	self._v_sock.quit()
	  except:
		pass # who cares if it is already closed


    #
    # Socket-less methods
    #

    def MessageCount(self):
	return len(self.uidDict)

